#!/usr/bin/env python2
# -*- coding: UTF-8 -*-

import os
import commands
import re
import ConfigParser
from snack import *

from graphicsmode import XRandr, gmMain

class Option(object):
    """
    Отдельная настройка
    :param name - имя настройки в секции section
    :param text - название настройки; specification - описание настройки
    """
    def __init__(self, name, type, section, text, specification, defaultValue, params = ''):
        self.name = name
        self.type = type
        self.section = section
        self.text = text
        self.specification = specification
        self.defaultValue = defaultValue
        self.params = params
        self.value = defaultValue

    def getValueDescription(self):
        if self.type == 'dict':
            return self.params[self.value]
        return self.value


class ScreenConfiguration(object):
    """
    Настройки экрана
    """
    def __init__(self, section):
        self.modeOption = Option('mode', 'list', section, 'Разрешение экрана', 'Разрешение экрана', '800x600', list())
        self.interfaceOption = Option('interface', 'list', section, 'Интерфейс',
                                      'Выбор интерфейса, к которому подключен экран', notAssigned, list())

        self.rotates = {'normal': 'Отсутствует', 'left': 'Влево', 'right': 'Вправо', 'inverted': 'Инвертировано'}
        self.reflects = {'normal': 'Отсутствует', 'x': 'Горизонтально', 'y': 'Вертикально',
                         'xy': 'Горизонтально и Вертикально'}
        self.scaleOption = Option('scale', 'entry', section, 'Изменение размера',
                                  'Изменение размера выводимого изображения.\nПо умолчанию 1.0x1.0',
                                  '1.0x1.0')
        self.reflectOption = Option('reflect', 'dict', section, 'Отражение', 'Отражение по вертикали или горизонтали',
                                    'normal', self.reflects)
        self.rotateOption = Option('rotate', 'dict', section, 'Поворот экрана', 'Настройка поворота экрана',
                                   'normal', self.rotates)
        self.gammaOption = Option('gamma', 'entry', section, 'Цветовая гамма',
                                  'Установка цветовой гаммы.\nПо умолчанию 1.0:1.0:1.0', '1.0:1.0:1.0')

        # при смене интерфейса экрана необходимо удалить секцию, к которой настройки относились изначально 
        self.initSection = section

    def getOptions(self):
        """
        Получения списка опций экрана
        """
        options = list()
        options.append(self.modeOption)
        options.append(self.scaleOption)
        options.append(self.reflectOption)
        options.append(self.rotateOption)
        options.append(self.gammaOption)
        return options

    def setModes(self, modes):
        """
        Установка доступных для экрана режимов
        """
        self.modeOption.params = modes

    def setInterfaces(self, interfaces):
        """
        Установка доступных для настройки интерфейсов
        """
        self.interfaceOption.params = interfaces

    def setSection(self, section):
        """
        Установка секции для настроек экрана
        """
        for option in self.getOptions():
            option.section = section

    def getAnnotation(self):
        """
        Строка для отображения в списке настроек
        """
        if self.interfaceOption.value == notAssigned:
            return "Не сконфигурирован"
        return "%-6s %s" % (self.interfaceOption.value, self.modeOption.value)


class Config(object):
    """
    Предоставляет настройки из ini файла
    """
    def __init__(self):
        self.iniFile = "/linuxcash/cash/conf/graphicsmode.ini"
        self.config = ConfigParser.RawConfigParser()

        # открытие конфига
        if os.path.isfile(self.iniFile):
            self.config.read(self.iniFile)
            self.checkMainSection()
        else:
            self.checkMainSection()
            self.write()

    def getBool(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.getboolean(section, option)
        except:
            pass
        return value

    def getString(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.get(section, option)
        except:
            pass
        return value

    def getOption(self, option):
        try:
            value = self.config.get(option.section, option.name)
        except:
            value = option.defaultValue
        return value

    def sections(self):
        return self.config.sections()

    def set(self, section, option, value):
        self.config.set(section, option, value)

    def setOption(self, option, value):
        if not value:
            self.config.remove_option(option.section, option.name)
        else:
            self.config.set(option.section, option.name, value)

    def checkMainSection(self):
        """
        Функция для проверки главной секции в конфиге.
        """
        if not self.config.has_section('main'):
            self.config.add_section('main')
        if self.getBool('main', 'setMode', False):
            self.config.set('main', 'setMode', 'True')
        else:
            self.config.set('main', 'setMode', 'False')

    def checkSection(self, section):
        """
        Функция для проверки секции в конфиге.
        """
        if not self.config.has_section(section):
            self.config.add_section(section)

    def removeSection(self, section):
        self.config.remove_section(section)

    def setScreenConfiguration(self, cfg):
        if cfg.initSection != notAssigned:
            self.removeSection(cfg.initSection)
        section = cfg.interfaceOption.value
        if section == notAssigned:
            return
        self.checkSection(section)
        cfg.initSection = section
        for option in cfg.getOptions():
            if option.value and option.value != notAssigned:
                self.setOption(option, option.value)

    def removeScreenSections(self):
        """
        Удаление всех секций с настройками экранов
        """
        sections = self.sections()
        sections.remove('main')
        for section in sections:
            self.removeSection(section)

    def removeLastScreenSection(self):
        """
        Удаление секции настроек второго экрана
        """
        sections = self.sections()
        sections.remove('main')
        if len(sections) == 2:
            self.removeSection(sections[1])

    def getUsedInterfaces(self):
        """
        Список используемых интерфейсов
        """
        result = list()
        screens = self.getScreenConfigurations()
        for screen in screens:
            result.append(screen.interfaceOption.value)
        return result

    def getScreenConfigurations(self):
        sections = self.sections()
        sections.remove('main')
        result = list()

        screens = 0
        for screenSection in sections:
            if screens == 2:
                break
            screenConfiguration = ScreenConfiguration(screenSection)
            screenConfiguration.interfaceOption.value = screenSection

            for option in screenConfiguration.getOptions():
                option.value = self.getOption(option)

            result.append(screenConfiguration)
            screens = screens + 1

        # миграция настроек
        if screens and not self.getString('main', 'screens', ''):
            self.set('main', 'screens', 'one')

        return result

    def write(self):
        """
        Запись конфигурации в файл 
        """
        with open(self.iniFile, 'wb') as configFile:
            self.config.write(configFile)

def listDlg(screen, params, current, title):
    """
    Функция показа диалогового окна со списком
    """
    l = len(params)
    height = l if l < 6 else 6
    scroll = 1 if l > 6 else 0
    li = Listbox(height = height, width = 28, scroll = scroll, returnExit = 1)
    if type(params) == list:
        for param in params:
            li.append(param, param)
    elif type(params) == dict:
        for key in params:
            li.append(params[key], key)

    g = GridForm(screen, title, 1, 1)
    g.add(li,0,0)
    g.addHotKey("ESC")

    try:
        li.setCurrent(current)
    except Exception, e:
        pass

    return li.current() if g.runOnce() == li else current

def entryDlg(screen,current,title):
    """
    Функция показа диалогового окна с полем ввода
    """
    entry = Entry(22)
    entry.set(current)
    bb = ButtonBar(screen, (("Применить", "ok"), ("Отмена", "canсel")), compact = 1)
    g = GridForm(screen, title, 1, 2)
    g.add(entry,0,0,(0,0,0,1))
    g.add(bb,0,1)
    g.addHotKey("ESC")
    g.addHotKey("ENTER")

    result = g.runOnce()
    return  entry.value() if bb.buttonPressed(result) == 'ok' or result == "ENTER" else current


def screenConfigurationDlg(cfg):
    """
    Функция показа диалогового окна с полем ввода
    :param cfg - конфигурация экрана ScreenConfiguration
    """
    buttons = ButtonBar(screen, [("Выход", "exit")], compact=1)
    textBox = Textbox(48, 3, '', wrap=1)
    form = GridForm(screen, "Настройки экрана 1", 20, 65)

    options = Listbox(8, width=48, returnExit=1)
    options.setCallback(callBack, [optionsList, textBox])

    form.addHotKey("ESC")
    form.add(options, 1, 1)
    form.add(textBox, 1, 2)
    form.add(buttons, 1, 3)

    modesTmp, allInterfaces = xRandr.getModes()
    modes = xRandr.getInterfacesModes()
    options.setCallback(callBack, [options, textBox])
    selectedOption = ''

    textBox.setText(cfg.interfaceOption.specification)

    availInterfaces = [notAssigned] + [i for i in allInterfaces if i not in config.getUsedInterfaces()]
    if cfg.interfaceOption.value not in availInterfaces:
        availInterfaces.append(cfg.interfaceOption.value)
    cfg.setInterfaces(availInterfaces)

    while True:
        options.clear()
        options.append(formatListLine(cfg.interfaceOption.text, cfg.interfaceOption.value), cfg.interfaceOption)

        if cfg.interfaceOption.value != notAssigned:
            cfg.setModes(modes[cfg.interfaceOption.value])
            cfg.setSection(cfg.interfaceOption.value)

            for option in cfg.getOptions():
                options.append(formatListLine(option.text, option.getValueDescription()), option)
        try:
            options.setCurrent(selectedOption)
        except Exception, e:
            pass

        result = form.run()
        selectedOption = options.current()

        if result == options:
            if selectedOption.type == 'list' or selectedOption.type == 'dict':
                selectedOption.value = listDlg(screen, selectedOption.params, selectedOption.value, selectedOption.text)
            elif selectedOption.type == 'entry':
                selectedOption.value = entryDlg(screen, selectedOption.value, selectedOption.text)
        elif buttons.buttonPressed(result) == 'exit' or result == "ESC":
            screen.popWindow()
            return cfg

def messageBox(title,text,buttons):
    """
    Функция показа диалогового окна с кнопками
    """
    bb = ButtonBar(screen, buttons, compact = 1)
    g = GridForm(screen, title, 1, 2)
    g.add(Label(text),0,0,(0,0,0,1))
    g.add(bb,0,1)
    return bb.buttonPressed(g.runOnce())

def callBack(obj):
    """
    функция для показа справки по определнному полю в списке настроек
    """
    li = obj[0]
    textBox = obj[1]
    textBox.setText("\n\n\n\n")
    textBox.setText(li.current().specification)

def formatListLine(text, value):
    text = text
    l = len(text)
    l = l / 2 + 1 if l & 1 else l / 2
    str = ' ' + text + ':' + (' ' * (20 - l)) + value
    return str

if __name__ == "__main__":
    notAssigned = 'Не назначено'
    config = Config()
    xRandr =  XRandr()
    modes, interfaces = xRandr.getModes()

    interfaces = [notAssigned] + interfaces
    mode = config.getString('main', 'mode', '800x600')
    if mode not in modes:
        config.set('main', 'mode', modes[0])

    # настройки для всех экранов
    boolItems = {'True': 'Да', 'False': 'Нет'}
    screensItems = {'all': 'Все', 'one': '1', 'two': '2'}

    setMode = Option('setMode', 'list', 'main', 'Изменять настройки', 'Если выключено, используется автоматическая настройка.',
                     'False', boolItems)
    screens = Option('screens', 'list', 'main', 'Используемые экраны', 'Количество используемых экранов',
                     'all', screensItems)

    screen1 = Option('screen', 'screen', str(), 'Экран 1', 'Настройки определенного экрана.', 'Не сконфигурирован')
    screen2 = Option('screen', 'screen', str(), 'Экран 2', 'Настройки определенного экрана.', 'Не сконфигурирован')

    mode = Option('mode', 'list', 'main', 'Разрешение экрана', 'Разрешение для всех мониторов', '800x600', modes)
    fb = Option('fb', 'entry', 'main', 'Размер скрина',
                      'Переконфигурация экрана до указанного размера.\nПо умолчанию равно разрешению экрана', '')

    # поиск подключеного интерфейса в конфиге
    screenSections = set(interfaces) & set(config.sections())

    # создание экрана
    screen = SnackScreen()
    screen.setColor('ROOT', 'white', 'brightmagenta')
    screen.setColor('HELPLINE', 'white', 'brightmagenta')

    # текст бокс для описаний опции
    textBox = Textbox(50, 4, '', wrap = 1)

    # список с настройками
    optionsList = Listbox(10, width = 50, returnExit = 1)
    optionsList.setCallback(callBack, [optionsList, textBox])

    # проверяем подлючены ли по SSH!
    ssh = True if 'SSH_TTY' in os.environ else False

    if ssh:
        bb = ButtonBar(screen, (('Применить (A)', 'apply'), ("Сохранить (S)", "save"), ("Выход (ESC)", "exit")), compact = 1)
    else:
        bb = ButtonBar(screen, (("Сохранить (S)", "save"), ("Выход (ESC)", "exit")), compact = 1)

    g = GridForm(screen, "Настройки экранов", 1, 4)

    g.add(optionsList, 0, 1)
    g.add(textBox, 0, 2)
    g.add(bb, 0, 3)

    # для trusty
    if not 's' in hotkeys:
        hotkeys['s'] = ord('s')
        hotkeys[ord('s')] = 's'
    if not 'a' in hotkeys:
        hotkeys['a'] = ord('a')
        hotkeys[ord('a')] = 'a'
    if not 'ENTER' in hotkeys:
        hotkeys['ENTER'] = 13
        hotkeys[13] = 'ENTER'

    # добавление горячих клавиш
    g.addHotKey("ESC")
    g.addHotKey('s')
    g.addHotKey('a')
    current = ''
    screenConfigurations = config.getScreenConfigurations()

    while True:
        # создаем список с опциями
        optionsList.clear()
        optionsList.append(formatListLine(setMode.text, boolItems[config.getOption(setMode)]), setMode)

        if config.getOption(setMode) == 'True':
            optionsList.append(formatListLine(screens.text, screensItems[config.getOption(screens)]), screens)
            if config.getOption(screens) == 'one' or config.getOption(screens) == 'two':
                if len(screenConfigurations) < 1:
                    screenConfigurations.append(ScreenConfiguration(notAssigned))
                optionsList.append(formatListLine("Экран 1", screenConfigurations[0].getAnnotation()), screen1)
                if config.getOption(screens) == 'two':
                    if len(screenConfigurations) < 2:
                        screenConfigurations.append(ScreenConfiguration(notAssigned))
                    optionsList.append(formatListLine("Экран 2", screenConfigurations[1].getAnnotation()), screen2)
            else:
                optionsList.append(formatListLine(mode.text, config.getOption(mode)), mode)
            optionsList.append(formatListLine(fb.text, config.getOption(fb)), fb)

        try:
            optionsList.setCurrent(current)
        except Exception, e:
            pass

        textBox.setText(optionsList.current().specification)
        # отображаем форму
        result = g.run()
        current = optionsList.current()
        resultBtn = bb.buttonPressed(result)

        # обработка резултатов выполнения
        if result == optionsList:
            if current == screens:
                val = listDlg(screen, current.params, config.getOption(current), current.text)
                # если настраиваются все мониторы - необходимо очистить конфигурацию от индивидуальных настроек
                if val == 'all':
                    screenConfigurations[0] = ScreenConfiguration(notAssigned)
                    screenConfigurations[1] = ScreenConfiguration(notAssigned)
                    config.removeScreenSections()
                elif val == 'one':
                    screenConfigurations[1] = ScreenConfiguration(notAssigned)
                    config.removeLastScreenSection()
                config.setOption(current, val)
            elif current.type == 'list':
                val = listDlg(screen, current.params, config.getOption(current), current.text)
                config.setOption(current, val)
            elif current.type == 'entry':
                val = entryDlg(screen, config.getOption(current), current.text)
                config.setOption(current, val)
            elif current == screen1:
                config.setScreenConfiguration(screenConfigurationDlg(screenConfigurations[0]))
            elif current == screen2:
                config.setScreenConfiguration(screenConfigurationDlg(screenConfigurations[1]))
        elif resultBtn == 'save' or result == "s":
            try:
                config.write()
            except Exception, e:
                messageBox('Ошибка сохранения', str(e), (("ОК", "ok"),))
            else:
                messageBox('Сохранено', 'Конфигурационный файл успешно сохранен!', (("ОК", "ok"),))
        elif (resultBtn == 'apply' or result == "a") and ssh:
            try:
                config.write()
                gmMain()
            except Exception, e:
                messageBox('Ошибка', str(e), (("ОК", "ok"),))
            else:
                messageBox('Выполнено', 'Настройки применены!', (("ОК", "ok"),))
        elif resultBtn == 'exit' or result == "ESC" :
            break

    screen.finish()
